@@ -20,8 +20,9 @@ import (
 	"fmt"
 	"strconv"
 
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/common/log"
 	"github.com/prometheus/procfs"
 )
 
@@ -30,8 +31,9 @@ const (
 )
 
 type buddyinfoCollector struct {
-	fs   procfs.FS
-	desc *prometheus.Desc
+	fs     procfs.FS
+	desc   *prometheus.Desc
+	logger log.Logger
 }
 
 func init() {
@@ -39,7 +41,7 @@ func init() {
 }
 
 // NewBuddyinfoCollector returns a new Collector exposing buddyinfo stats.
-func NewBuddyinfoCollector() (Collector, error) {
+func NewBuddyinfoCollector(logger log.Logger) (Collector, error) {
 	desc := prometheus.NewDesc(
 		prometheus.BuildFQName(namespace, buddyInfoSubsystem, "blocks"),
 		"Count of free blocks according to size.",
@@ -47,20 +49,20 @@ func NewBuddyinfoCollector() (Collector, error) {
 	)
 	fs, err := procfs.NewFS(*procPath)
 	if err != nil {
-		return nil, fmt.Errorf("failed to open procfs: %v", err)
+		return nil, fmt.Errorf("failed to open procfs: %w", err)
 	}
-	return &buddyinfoCollector{fs, desc}, nil
+	return &buddyinfoCollector{fs, desc, logger}, nil
 }
 
 // Update calls (*buddyinfoCollector).getBuddyInfo to get the platform specific
 // buddyinfo metrics.
 func (c *buddyinfoCollector) Update(ch chan<- prometheus.Metric) error {
-	buddyInfo, err := c.fs.NewBuddyInfo()
+	buddyInfo, err := c.fs.BuddyInfo()
 	if err != nil {
-		return fmt.Errorf("couldn't get buddyinfo: %s", err)
+		return fmt.Errorf("couldn't get buddyinfo: %w", err)
 	}
 
-	log.Debugf("Set node_buddy: %#v", buddyInfo)
+	level.Debug(c.logger).Log("msg", "Set node_buddy", "buddyInfo", buddyInfo)
 	for _, entry := range buddyInfo {
 		for size, value := range entry.Sizes {
 			ch <- prometheus.MustNewConstMetric(
