@@ -26,7 +26,7 @@ const (
 var (
 	nodeCPUSecondsDesc = prometheus.NewDesc(
 		prometheus.BuildFQName(namespace, cpuCollectorSubsystem, "seconds_total"),
-		"Seconds the cpus spent in each mode.",
+		"Seconds the CPUs spent in each mode.",
 		[]string{"cpu", "mode"}, nil,
 	)
 )
