@@ -19,6 +19,8 @@ package collector
 import (
 	"regexp"
 
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
 	"gopkg.in/alecthomas/kingpin.v2"
 )
@@ -48,6 +50,7 @@ type filesystemCollector struct {
 	sizeDesc, freeDesc, availDesc *prometheus.Desc
 	filesDesc, filesFreeDesc      *prometheus.Desc
 	roDesc, deviceErrorDesc       *prometheus.Desc
+	logger                        log.Logger
 }
 
 type filesystemLabels struct {
@@ -66,9 +69,11 @@ func init() {
 }
 
 // NewFilesystemCollector returns a new Collector exposing filesystems stats.
-func NewFilesystemCollector() (Collector, error) {
+func NewFilesystemCollector(logger log.Logger) (Collector, error) {
 	subsystem := "filesystem"
+	level.Info(logger).Log("msg", "Parsed flag --collector.filesystem.ignored-mount-points", "flag", *ignoredMountPoints)
 	mountPointPattern := regexp.MustCompile(*ignoredMountPoints)
+	level.Info(logger).Log("msg", "Parsed flag --collector.filesystem.ignored-fs-types", "flag", *ignoredFSTypes)
 	filesystemsTypesPattern := regexp.MustCompile(*ignoredFSTypes)
 
 	sizeDesc := prometheus.NewDesc(
@@ -123,6 +128,7 @@ func NewFilesystemCollector() (Collector, error) {
 		filesFreeDesc:             filesFreeDesc,
 		roDesc:                    roDesc,
 		deviceErrorDesc:           deviceErrorDesc,
+		logger:                    logger,
 	}, nil
 }
 
