@@ -19,15 +19,15 @@ import (
 	"errors"
 	"io/ioutil"
 	"os"
-	"path"
 	"path/filepath"
 	"regexp"
 	"strconv"
 	"strings"
-	"syscall"
 
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/common/log"
+	"golang.org/x/sys/unix"
 )
 
 var (
@@ -46,12 +46,14 @@ func init() {
 	registerCollector("hwmon", defaultEnabled, NewHwMonCollector)
 }
 
-type hwMonCollector struct{}
+type hwMonCollector struct {
+	logger log.Logger
+}
 
 // NewHwMonCollector returns a new Collector exposing /sys/class/hwmon stats
 // (similar to lm-sensors).
-func NewHwMonCollector() (Collector, error) {
-	return &hwMonCollector{}, nil
+func NewHwMonCollector(logger log.Logger) (Collector, error) {
+	return &hwMonCollector{logger}, nil
 }
 
 func cleanMetricName(name string) string {
@@ -87,9 +89,9 @@ func sysReadFile(file string) ([]byte, error) {
 	// Go's ioutil.ReadFile implementation to poll forever.
 	//
 	// Since we either want to read data or bail immediately, do the simplest
-	// possible read using syscall directly.
+	// possible read using system call directly.
 	b := make([]byte, 128)
-	n, err := syscall.Read(int(f.Fd()), b)
+	n, err := unix.Read(int(f.Fd()), b)
 	if err != nil {
 		return nil, err
 	}
@@ -138,7 +140,7 @@ func collectSensorData(dir string, data map[string]map[string]string) error {
 
 		for _, t := range hwmonSensorTypes {
 			if t == sensorType {
-				addValueFile(data, sensorType+strconv.Itoa(sensorNum), sensorProperty, path.Join(dir, file.Name()))
+				addValueFile(data, sensorType+strconv.Itoa(sensorNum), sensorProperty, filepath.Join(dir, file.Name()))
 				break
 			}
 		}
@@ -157,8 +159,8 @@ func (c *hwMonCollector) updateHwmon(ch chan<- prometheus.Metric, dir string) er
 	if err != nil {
 		return err
 	}
-	if _, err := os.Stat(path.Join(dir, "device")); err == nil {
-		err := collectSensorData(path.Join(dir, "device"), data)
+	if _, err := os.Stat(filepath.Join(dir, "device")); err == nil {
+		err := collectSensorData(filepath.Join(dir, "device"), data)
 		if err != nil {
 			return err
 		}
@@ -274,6 +276,9 @@ func (c *hwMonCollector) updateHwmon(ch chan<- prometheus.Metric, dir string) er
 				continue
 			}
 			if sensorType == "temp" && element != "type" {
+				if element == "" {
+					element = "input"
+				}
 				desc := prometheus.NewDesc(name+"_celsius", "Hardware monitor for temperature ("+element+")", hwmonLabelDesc, nil)
 				ch <- prometheus.MustNewConstMetric(
 					desc, prometheus.GaugeValue, parsedValue*0.001, labels...)
@@ -350,10 +355,10 @@ func (c *hwMonCollector) hwmonName(dir string) (string, error) {
 
 	// preference 1: construct a name based on device name, always unique
 
-	devicePath, devErr := filepath.EvalSymlinks(path.Join(dir, "device"))
+	devicePath, devErr := filepath.EvalSymlinks(filepath.Join(dir, "device"))
 	if devErr == nil {
-		devPathPrefix, devName := path.Split(devicePath)
-		_, devType := path.Split(strings.TrimRight(devPathPrefix, "/"))
+		devPathPrefix, devName := filepath.Split(devicePath)
+		_, devType := filepath.Split(strings.TrimRight(devPathPrefix, "/"))
 
 		cleanDevName := cleanMetricName(devName)
 		cleanDevType := cleanMetricName(devType)
@@ -368,7 +373,7 @@ func (c *hwMonCollector) hwmonName(dir string) (string, error) {
 	}
 
 	// preference 2: is there a name file
-	sysnameRaw, nameErr := ioutil.ReadFile(path.Join(dir, "name"))
+	sysnameRaw, nameErr := ioutil.ReadFile(filepath.Join(dir, "name"))
 	if nameErr == nil && string(sysnameRaw) != "" {
 		cleanName := cleanMetricName(string(sysnameRaw))
 		if cleanName != "" {
@@ -385,7 +390,7 @@ func (c *hwMonCollector) hwmonName(dir string) (string, error) {
 	}
 
 	// take the last path element, this will be hwmonX
-	_, name := path.Split(realDir)
+	_, name := filepath.Split(realDir)
 	cleanName := cleanMetricName(name)
 	if cleanName != "" {
 		return cleanName, nil
@@ -396,7 +401,7 @@ func (c *hwMonCollector) hwmonName(dir string) (string, error) {
 // hwmonHumanReadableChipName is similar to the methods in hwmonName, but with
 // different precedences -- we can allow duplicates here.
 func (c *hwMonCollector) hwmonHumanReadableChipName(dir string) (string, error) {
-	sysnameRaw, nameErr := ioutil.ReadFile(path.Join(dir, "name"))
+	sysnameRaw, nameErr := ioutil.ReadFile(filepath.Join(dir, "name"))
 	if nameErr != nil {
 		return "", nameErr
 	}
@@ -415,20 +420,20 @@ func (c *hwMonCollector) Update(ch chan<- prometheus.Metric) error {
 	// Step 1: scan /sys/class/hwmon, resolve all symlinks and call
 	//         updatesHwmon for each folder
 
-	hwmonPathName := path.Join(sysFilePath("class"), "hwmon")
+	hwmonPathName := filepath.Join(sysFilePath("class"), "hwmon")
 
 	hwmonFiles, err := ioutil.ReadDir(hwmonPathName)
 	if err != nil {
-		if os.IsNotExist(err) {
-			log.Debug("hwmon collector metrics are not available for this system")
-			return nil
+		if errors.Is(err, os.ErrNotExist) {
+			level.Debug(c.logger).Log("msg", "hwmon collector metrics are not available for this system")
+			return ErrNoData
 		}
 
 		return err
 	}
 
 	for _, hwDir := range hwmonFiles {
-		hwmonXPathName := path.Join(hwmonPathName, hwDir.Name())
+		hwmonXPathName := filepath.Join(hwmonPathName, hwDir.Name())
 
 		if hwDir.Mode()&os.ModeSymlink > 0 {
 			hwDir, err = os.Stat(hwmonXPathName)
