@@ -17,8 +17,9 @@ package collector
 
 import (
 	"fmt"
-	"path"
+	"path/filepath"
 
+	"github.com/go-kit/kit/log"
 	"github.com/prometheus/client_golang/prometheus"
 )
 
@@ -29,6 +30,7 @@ var (
 
 type ksmdCollector struct {
 	metricDescs map[string]*prometheus.Desc
+	logger      log.Logger
 }
 
 func init() {
@@ -47,7 +49,7 @@ func getCanonicalMetricName(filename string) string {
 }
 
 // NewKsmdCollector returns a new Collector exposing kernel/system statistics.
-func NewKsmdCollector() (Collector, error) {
+func NewKsmdCollector(logger log.Logger) (Collector, error) {
 	subsystem := "ksmd"
 	descs := make(map[string]*prometheus.Desc)
 
@@ -56,13 +58,13 @@ func NewKsmdCollector() (Collector, error) {
 			prometheus.BuildFQName(namespace, subsystem, getCanonicalMetricName(n)),
 			fmt.Sprintf("ksmd '%s' file.", n), nil, nil)
 	}
-	return &ksmdCollector{descs}, nil
+	return &ksmdCollector{descs, logger}, nil
 }
 
 // Update implements Collector and exposes kernel and system statistics.
 func (c *ksmdCollector) Update(ch chan<- prometheus.Metric) error {
 	for _, n := range ksmdFiles {
-		val, err := readUintFromFile(sysFilePath(path.Join("kernel/mm/ksm", n)))
+		val, err := readUintFromFile(sysFilePath(filepath.Join("kernel/mm/ksm", n)))
 		if err != nil {
 			return err
 		}
