@@ -16,11 +16,15 @@
 package collector
 
 import (
+	"errors"
 	"fmt"
 	"os"
+	"strings"
+	"syscall"
 
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/common/log"
 	"github.com/prometheus/procfs"
 )
 
@@ -31,6 +35,7 @@ type processCollector struct {
 	procsState  *prometheus.Desc
 	pidUsed     *prometheus.Desc
 	pidMax      *prometheus.Desc
+	logger      log.Logger
 }
 
 func init() {
@@ -38,10 +43,10 @@ func init() {
 }
 
 // NewProcessStatCollector returns a new Collector exposing process data read from the proc filesystem.
-func NewProcessStatCollector() (Collector, error) {
+func NewProcessStatCollector(logger log.Logger) (Collector, error) {
 	fs, err := procfs.NewFS(*procPath)
 	if err != nil {
-		return nil, fmt.Errorf("failed to open procfs: %v", err)
+		return nil, fmt.Errorf("failed to open procfs: %w", err)
 	}
 	subsystem := "processes"
 	return &processCollector{
@@ -67,18 +72,19 @@ func NewProcessStatCollector() (Collector, error) {
 		pidMax: prometheus.NewDesc(prometheus.BuildFQName(namespace, subsystem, "max_processes"),
 			"Number of max PIDs limit", nil, nil,
 		),
+		logger: logger,
 	}, nil
 }
 func (c *processCollector) Update(ch chan<- prometheus.Metric) error {
 	pids, states, threads, err := c.getAllocatedThreads()
 	if err != nil {
-		return fmt.Errorf("unable to retrieve number of allocated threads: %q", err)
+		return fmt.Errorf("unable to retrieve number of allocated threads: %w", err)
 	}
 
 	ch <- prometheus.MustNewConstMetric(c.threadAlloc, prometheus.GaugeValue, float64(threads))
 	maxThreads, err := readUintFromFile(procFilePath("sys/kernel/threads-max"))
 	if err != nil {
-		return fmt.Errorf("unable to retrieve limit number of threads: %q", err)
+		return fmt.Errorf("unable to retrieve limit number of threads: %w", err)
 	}
 	ch <- prometheus.MustNewConstMetric(c.threadLimit, prometheus.GaugeValue, float64(maxThreads))
 
@@ -88,7 +94,7 @@ func (c *processCollector) Update(ch chan<- prometheus.Metric) error {
 
 	pidM, err := readUintFromFile(procFilePath("sys/kernel/pid_max"))
 	if err != nil {
-		return fmt.Errorf("unable to retrieve limit number of maximum pids alloved: %q", err)
+		return fmt.Errorf("unable to retrieve limit number of maximum pids alloved: %w", err)
 	}
 	ch <- prometheus.MustNewConstMetric(c.pidUsed, prometheus.GaugeValue, float64(pids))
 	ch <- prometheus.MustNewConstMetric(c.pidMax, prometheus.GaugeValue, float64(pidM))
@@ -105,13 +111,14 @@ func (c *processCollector) getAllocatedThreads() (int, map[string]int32, int, er
 	thread := 0
 	procStates := make(map[string]int32)
 	for _, pid := range p {
-		stat, err := pid.NewStat()
-		// PIDs can vanish between getting the list and getting stats.
-		if os.IsNotExist(err) {
-			log.Debugf("file not found when retrieving stats: %q", err)
-			continue
-		}
+		stat, err := pid.Stat()
 		if err != nil {
+			// PIDs can vanish between getting the list and getting stats.
+			if errors.Is(err, os.ErrNotExist) || strings.Contains(err.Error(), syscall.ESRCH.Error()) {
+				level.Debug(c.logger).Log("msg", "file not found when retrieving stats for pid", "pid", pid, "err", err)
+				continue
+			}
+			level.Debug(c.logger).Log("msg", "error reading stat for pid", "pid", pid, "err", err)
 			return 0, nil, 0, err
 		}
 		pids++
