@@ -18,6 +18,7 @@ package collector
 import (
 	"testing"
 
+	"github.com/go-kit/kit/log"
 	"github.com/prometheus/procfs"
 	kingpin "gopkg.in/alecthomas/kingpin.v2"
 )
@@ -31,7 +32,7 @@ func TestReadProcessStatus(t *testing.T) {
 	if err != nil {
 		t.Errorf("failed to open procfs: %v", err)
 	}
-	c := processCollector{fs: fs}
+	c := processCollector{fs: fs, logger: log.NewNopLogger()}
 	pids, states, threads, err := c.getAllocatedThreads()
 	if err != nil {
 		t.Fatalf("Cannot retrieve data from procfs getAllocatedThreads function: %v ", err)
