@@ -14,6 +14,7 @@
 package collector
 
 import (
+	"github.com/go-kit/kit/log"
 	"regexp"
 	"testing"
 
@@ -88,23 +89,24 @@ func getUnitListFixtures() [][]unit {
 
 func TestSystemdIgnoreFilter(t *testing.T) {
 	fixtures := getUnitListFixtures()
-	whitelistPattern := regexp.MustCompile("^foo$")
-	blacklistPattern := regexp.MustCompile("^bar$")
-	filtered := filterUnits(fixtures[0], whitelistPattern, blacklistPattern)
+	includePattern := regexp.MustCompile("^foo$")
+	excludePattern := regexp.MustCompile("^bar$")
+	filtered := filterUnits(fixtures[0], includePattern, excludePattern, log.NewNopLogger())
 	for _, unit := range filtered {
-		if blacklistPattern.MatchString(unit.Name) || !whitelistPattern.MatchString(unit.Name) {
+		if excludePattern.MatchString(unit.Name) || !includePattern.MatchString(unit.Name) {
 			t.Error(unit.Name, "should not be in the filtered list")
 		}
 	}
 }
 func TestSystemdIgnoreFilterDefaultKeepsAll(t *testing.T) {
-	c, err := NewSystemdCollector()
+	logger := log.NewNopLogger()
+	c, err := NewSystemdCollector(logger)
 	if err != nil {
 		t.Fatal(err)
 	}
 	fixtures := getUnitListFixtures()
 	collector := c.(*systemdCollector)
-	filtered := filterUnits(fixtures[0], collector.unitWhitelistPattern, collector.unitBlacklistPattern)
+	filtered := filterUnits(fixtures[0], collector.unitIncludePattern, collector.unitExcludePattern, logger)
 	// Adjust fixtures by 3 "not-found" units.
 	if len(filtered) != len(fixtures[0])-3 {
 		t.Error("Default filters removed units")
