@@ -16,21 +16,25 @@
 
 package collector
 
-// #include <sys/timex.h>
-import "C"
-
 import (
+	"errors"
 	"fmt"
-	"syscall"
+	"os"
 
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
 	"github.com/prometheus/client_golang/prometheus"
+	"golang.org/x/sys/unix"
 )
 
 const (
-	// The system clock is not synchronized to a reliable server.
-	timeError = C.TIME_ERROR
-	// The timex.Status time resolution bit, 0 = microsecond, 1 = nanoseconds.
-	staNano = C.STA_NANO
+	// The system clock is not synchronized to a reliable
+	// server (TIME_ERROR).
+	timeError = 5
+	// The timex.Status time resolution bit (STA_NANO),
+	// 0 = microsecond, 1 = nanoseconds.
+	staNano = 0x2000
+
 	// 1 second in
 	nanoSeconds  = 1000000000
 	microSeconds = 1000000
@@ -54,6 +58,7 @@ type timexCollector struct {
 	stbcnt,
 	tai,
 	syncStatus typedDesc
+	logger log.Logger
 }
 
 func init() {
@@ -61,7 +66,7 @@ func init() {
 }
 
 // NewTimexCollector returns a new Collector exposing adjtime(3) stats.
-func NewTimexCollector() (Collector, error) {
+func NewTimexCollector(logger log.Logger) (Collector, error) {
 	const subsystem = "timex"
 
 	return &timexCollector{
@@ -150,17 +155,22 @@ func NewTimexCollector() (Collector, error) {
 			"Is clock synchronized to a reliable server (1 = yes, 0 = no).",
 			nil, nil,
 		), prometheus.GaugeValue},
+		logger: logger,
 	}, nil
 }
 
 func (c *timexCollector) Update(ch chan<- prometheus.Metric) error {
 	var syncStatus float64
 	var divisor float64
-	var timex = new(syscall.Timex)
+	var timex = new(unix.Timex)
 
-	status, err := syscall.Adjtimex(timex)
+	status, err := unix.Adjtimex(timex)
 	if err != nil {
-		return fmt.Errorf("failed to retrieve adjtimex stats: %v", err)
+		if errors.Is(err, os.ErrPermission) {
+			level.Debug(c.logger).Log("msg", "Not collecting timex metrics", "err", err)
+			return ErrNoData
+		}
+		return fmt.Errorf("failed to retrieve adjtimex stats: %w", err)
 	}
 
 	if status == timeError {
