@@ -11,18 +11,22 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+// +build !nowifi
+
 package collector
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io/ioutil"
 	"os"
 	"path/filepath"
 
+	"github.com/go-kit/kit/log"
+	"github.com/go-kit/kit/log/level"
 	"github.com/mdlayher/wifi"
 	"github.com/prometheus/client_golang/prometheus"
-	"github.com/prometheus/common/log"
 	"gopkg.in/alecthomas/kingpin.v2"
 )
 
@@ -34,10 +38,14 @@ type wifiCollector struct {
 	stationInactiveSeconds       *prometheus.Desc
 	stationReceiveBitsPerSecond  *prometheus.Desc
 	stationTransmitBitsPerSecond *prometheus.Desc
+	stationReceiveBytesTotal     *prometheus.Desc
+	stationTransmitBytesTotal    *prometheus.Desc
 	stationSignalDBM             *prometheus.Desc
 	stationTransmitRetriesTotal  *prometheus.Desc
 	stationTransmitFailedTotal   *prometheus.Desc
 	stationBeaconLossTotal       *prometheus.Desc
+
+	logger log.Logger
 }
 
 var (
@@ -45,7 +53,7 @@ var (
 )
 
 func init() {
-	registerCollector("wifi", defaultEnabled, NewWifiCollector)
+	registerCollector("wifi", defaultDisabled, NewWifiCollector)
 }
 
 var _ wifiStater = &wifi.Client{}
@@ -55,24 +63,24 @@ type wifiStater interface {
 	BSS(ifi *wifi.Interface) (*wifi.BSS, error)
 	Close() error
 	Interfaces() ([]*wifi.Interface, error)
-	StationInfo(ifi *wifi.Interface) (*wifi.StationInfo, error)
+	StationInfo(ifi *wifi.Interface) ([]*wifi.StationInfo, error)
 }
 
 // NewWifiCollector returns a new Collector exposing Wifi statistics.
-func NewWifiCollector() (Collector, error) {
+func NewWifiCollector(logger log.Logger) (Collector, error) {
 	const (
 		subsystem = "wifi"
 	)
 
 	var (
-		labels = []string{"device"}
+		labels = []string{"device", "mac_address"}
 	)
 
 	return &wifiCollector{
 		interfaceFrequencyHertz: prometheus.NewDesc(
 			prometheus.BuildFQName(namespace, subsystem, "interface_frequency_hertz"),
 			"The current frequency a WiFi interface is operating at, in hertz.",
-			labels,
+			[]string{"device"},
 			nil,
 		),
 
@@ -111,6 +119,20 @@ func NewWifiCollector() (Collector, error) {
 			nil,
 		),
 
+		stationReceiveBytesTotal: prometheus.NewDesc(
+			prometheus.BuildFQName(namespace, subsystem, "station_receive_bytes_total"),
+			"The total number of bytes received by a WiFi station.",
+			labels,
+			nil,
+		),
+
+		stationTransmitBytesTotal: prometheus.NewDesc(
+			prometheus.BuildFQName(namespace, subsystem, "station_transmit_bytes_total"),
+			"The total number of bytes transmitted by a WiFi station.",
+			labels,
+			nil,
+		),
+
 		stationSignalDBM: prometheus.NewDesc(
 			prometheus.BuildFQName(namespace, subsystem, "station_signal_dbm"),
 			"The current WiFi signal strength, in decibel-milliwatts (dBm).",
@@ -138,6 +160,7 @@ func NewWifiCollector() (Collector, error) {
 			labels,
 			nil,
 		),
+		logger: logger,
 	}, nil
 }
 
@@ -145,22 +168,22 @@ func (c *wifiCollector) Update(ch chan<- prometheus.Metric) error {
 	stat, err := newWifiStater(*collectorWifi)
 	if err != nil {
 		// Cannot access wifi metrics, report no error.
-		if os.IsNotExist(err) {
-			log.Debug("wifi collector metrics are not available for this system")
-			return nil
+		if errors.Is(err, os.ErrNotExist) {
+			level.Debug(c.logger).Log("msg", "wifi collector metrics are not available for this system")
+			return ErrNoData
 		}
-		if os.IsPermission(err) {
-			log.Debug("wifi collector got permission denied when accessing metrics")
-			return nil
+		if errors.Is(err, os.ErrPermission) {
+			level.Debug(c.logger).Log("msg", "wifi collector got permission denied when accessing metrics")
+			return ErrNoData
 		}
 
-		return fmt.Errorf("failed to access wifi data: %v", err)
+		return fmt.Errorf("failed to access wifi data: %w", err)
 	}
 	defer stat.Close()
 
 	ifis, err := stat.Interfaces()
 	if err != nil {
-		return fmt.Errorf("failed to retrieve wifi interfaces: %v", err)
+		return fmt.Errorf("failed to retrieve wifi interfaces: %w", err)
 	}
 
 	for _, ifi := range ifis {
@@ -169,7 +192,7 @@ func (c *wifiCollector) Update(ch chan<- prometheus.Metric) error {
 			continue
 		}
 
-		log.Debugf("probing wifi device %q with type %q", ifi.Name, ifi.Type)
+		level.Debug(c.logger).Log("msg", "probing wifi device with type", "wifi", ifi.Name, "type", ifi.Type)
 
 		ch <- prometheus.MustNewConstMetric(
 			c.interfaceFrequencyHertz,
@@ -179,26 +202,28 @@ func (c *wifiCollector) Update(ch chan<- prometheus.Metric) error {
 		)
 
 		// When a statistic is not available for a given interface, package wifi
-		// returns an error compatible with os.IsNotExist.  We leverage this to
-		// only export metrics which are actually valid for given interface types.
+		// returns a os.ErrNotExist error.  We leverage this to only export
+		// metrics which are actually valid for given interface types.
 
 		bss, err := stat.BSS(ifi)
 		switch {
 		case err == nil:
 			c.updateBSSStats(ch, ifi.Name, bss)
-		case os.IsNotExist(err):
-			log.Debugf("BSS information not found for wifi device %q", ifi.Name)
+		case errors.Is(err, os.ErrNotExist):
+			level.Debug(c.logger).Log("msg", "BSS information not found for wifi device", "name", ifi.Name)
 		default:
 			return fmt.Errorf("failed to retrieve BSS for device %s: %v",
 				ifi.Name, err)
 		}
 
-		info, err := stat.StationInfo(ifi)
+		stations, err := stat.StationInfo(ifi)
 		switch {
 		case err == nil:
-			c.updateStationStats(ch, ifi.Name, info)
-		case os.IsNotExist(err):
-			log.Debugf("station information not found for wifi device %q", ifi.Name)
+			for _, station := range stations {
+				c.updateStationStats(ch, ifi.Name, station)
+			}
+		case errors.Is(err, os.ErrNotExist):
+			level.Debug(c.logger).Log("msg", "station information not found for wifi device", "name", ifi.Name)
 		default:
 			return fmt.Errorf("failed to retrieve station info for device %q: %v",
 				ifi.Name, err)
@@ -227,6 +252,7 @@ func (c *wifiCollector) updateStationStats(ch chan<- prometheus.Metric, device s
 		prometheus.CounterValue,
 		info.Connected.Seconds(),
 		device,
+		info.HardwareAddr.String(),
 	)
 
 	ch <- prometheus.MustNewConstMetric(
@@ -234,6 +260,7 @@ func (c *wifiCollector) updateStationStats(ch chan<- prometheus.Metric, device s
 		prometheus.GaugeValue,
 		info.Inactive.Seconds(),
 		device,
+		info.HardwareAddr.String(),
 	)
 
 	ch <- prometheus.MustNewConstMetric(
@@ -241,6 +268,7 @@ func (c *wifiCollector) updateStationStats(ch chan<- prometheus.Metric, device s
 		prometheus.GaugeValue,
 		float64(info.ReceiveBitrate),
 		device,
+		info.HardwareAddr.String(),
 	)
 
 	ch <- prometheus.MustNewConstMetric(
@@ -248,6 +276,23 @@ func (c *wifiCollector) updateStationStats(ch chan<- prometheus.Metric, device s
 		prometheus.GaugeValue,
 		float64(info.TransmitBitrate),
 		device,
+		info.HardwareAddr.String(),
+	)
+
+	ch <- prometheus.MustNewConstMetric(
+		c.stationReceiveBytesTotal,
+		prometheus.CounterValue,
+		float64(info.ReceivedBytes),
+		device,
+		info.HardwareAddr.String(),
+	)
+
+	ch <- prometheus.MustNewConstMetric(
+		c.stationTransmitBytesTotal,
+		prometheus.CounterValue,
+		float64(info.TransmittedBytes),
+		device,
+		info.HardwareAddr.String(),
 	)
 
 	ch <- prometheus.MustNewConstMetric(
@@ -255,6 +300,7 @@ func (c *wifiCollector) updateStationStats(ch chan<- prometheus.Metric, device s
 		prometheus.GaugeValue,
 		float64(info.Signal),
 		device,
+		info.HardwareAddr.String(),
 	)
 
 	ch <- prometheus.MustNewConstMetric(
@@ -262,6 +308,7 @@ func (c *wifiCollector) updateStationStats(ch chan<- prometheus.Metric, device s
 		prometheus.CounterValue,
 		float64(info.TransmitRetries),
 		device,
+		info.HardwareAddr.String(),
 	)
 
 	ch <- prometheus.MustNewConstMetric(
@@ -269,6 +316,7 @@ func (c *wifiCollector) updateStationStats(ch chan<- prometheus.Metric, device s
 		prometheus.CounterValue,
 		float64(info.TransmitFailed),
 		device,
+		info.HardwareAddr.String(),
 	)
 
 	ch <- prometheus.MustNewConstMetric(
@@ -276,6 +324,7 @@ func (c *wifiCollector) updateStationStats(ch chan<- prometheus.Metric, device s
 		prometheus.CounterValue,
 		float64(info.BeaconLoss),
 		device,
+		info.HardwareAddr.String(),
 	)
 }
 
@@ -346,13 +395,13 @@ func (s *mockWifiStater) Interfaces() ([]*wifi.Interface, error) {
 	return ifis, nil
 }
 
-func (s *mockWifiStater) StationInfo(ifi *wifi.Interface) (*wifi.StationInfo, error) {
+func (s *mockWifiStater) StationInfo(ifi *wifi.Interface) ([]*wifi.StationInfo, error) {
 	p := filepath.Join(ifi.Name, "stationinfo.json")
 
-	var info wifi.StationInfo
-	if err := s.unmarshalJSONFile(p, &info); err != nil {
+	var stations []*wifi.StationInfo
+	if err := s.unmarshalJSONFile(p, &stations); err != nil {
 		return nil, err
 	}
 
-	return &info, nil
+	return stations, nil
 }
