@@ -5,6 +5,7 @@ set -euf -o pipefail
 enabled_collectors=$(cat << COLLECTORS
   arp
   bcache
+  btrfs
   buddyinfo
   conntrack
   cpu
@@ -13,6 +14,7 @@ enabled_collectors=$(cat << COLLECTORS
   drbd
   edac
   entropy
+  fibrechannel
   filefd
   hwmon
   infiniband
@@ -30,15 +32,20 @@ enabled_collectors=$(cat << COLLECTORS
   nfsd
   pressure
   qdisc
+  rapl
+  schedstat
   sockstat
   stat
+  thermal_zone
   textfile
   bonding
+  udp_queues 
   vmstat
   wifi
   xfs
   zfs
   processes
+  zoneinfo
 COLLECTORS
 )
 disabled_collectors=$(cat << COLLECTORS
@@ -101,6 +108,10 @@ fi
   --collector.wifi.fixtures="collector/fixtures/wifi" \
   --collector.qdisc.fixtures="collector/fixtures/qdisc/" \
   --collector.netclass.ignored-devices="(bond0|dmz|int)" \
+  --collector.bcache.priorityStats \
+  --collector.cpu.info \
+  --collector.cpu.info.flags-include="^(aes|avx.?|constant_tsc)$" \
+  --collector.cpu.info.bugs-include="^(cpu_meltdown|spectre_.*|mds)$" \
   --web.listen-address "127.0.0.1:${port}" \
   --log.level="debug" > "${tmpdir}/node_exporter.log" 2>&1 &
 
